/*
 * $Id: rb_smart.c 68 2004-08-16 15:42:19Z tilman $
 *
 * Copyright (C) 2004 Tilman Sauerbeck (tilman at code-monkey de)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ruby.h>

#include <Evas.h>

#include "rb_evas_main.h"
#include "rb_evas.h"
#include "rb_evas_object.h"

#define RUBY_EVAS_SMART_OBJECT_KEY "__RB_EVAS_OBJECT_SMART_OBJECT"

#define GET_EVAS_OBJECT(obj, o) \
	VALUE (obj); \
\
	obj = (VALUE) evas_object_data_get (o, \
	                                    RUBY_EVAS_SMART_OBJECT_KEY); \
	if (!obj) { \
		rb_raise (rb_eException, \
		          "EvasObject Ruby object key missing"); \
		return; \
	}

#define SMART_CB_BODY(name) \
	static ID id; \
\
	GET_EVAS_OBJECT (self, o); \
\
	if (!id) \
		id = rb_intern ("on_"#name); \
\
	if (!rb_respond_to (self, id)) \
		return;

#define SMART_CB(name) \
	static void on_##name (Evas_Object *o) \
	{ \
		SMART_CB_BODY (name); \
		rb_funcall (self, id, 0); \
	}

#define SMART_CB_OBJ(name) \
	static void on_##name (Evas_Object *o, Evas_Object *other) \
	{ \
		SMART_CB_BODY (name); \
		rb_funcall (self, id, 1, TO_EVAS_OBJECT (other)); \
	}

#define SMART_CB_COORD(name) \
	static void on_##name (Evas_Object *o, Evas_Coord a, Evas_Coord b) \
	{ \
		SMART_CB_BODY (name); \
		rb_funcall (self, id, 2, INT2FIX ((int) a), INT2FIX ((int) b)); \
	}

SMART_CB (delete);
SMART_CB (raise);
SMART_CB (lower);
SMART_CB (show);
SMART_CB (hide);
SMART_CB (clip_unset);
SMART_CB_OBJ (stack_above);
SMART_CB_OBJ (stack_below);
SMART_CB_OBJ (clip_set);
SMART_CB_COORD (move);
SMART_CB_COORD (resize);

static void on_layer_set (Evas_Object *o, int layer)
{
	SMART_CB_BODY (layer_set);

	rb_funcall (self, id, 1, FIX2INT (layer));
}

static void on_color_set (Evas_Object *o, int r, int g, int b, int a)
{
	SMART_CB_BODY (color_set);

	rb_funcall (self, id, 4, INT2FIX (r), INT2FIX (g), INT2FIX (b),
	            INT2FIX (a));
}

/*
 * call-seq:
 *  Evas::Smart.new(name) => smart
 *
 * Creates a Evas::Smart object with the given name.
 */
static VALUE c_new (VALUE klass, VALUE name)
{
	VALUE self;
	Evas_Smart **smart;

	Check_Type (name, T_STRING);

	self = Data_Make_Struct (klass, Evas_Smart *, NULL, free, smart);
	*smart = evas_smart_new (StringValuePtr (name),
	                         NULL, on_delete, on_layer_set,
	                         on_raise, on_lower, on_stack_above,
	                         on_stack_below, on_move, on_resize,
	                         on_show, on_hide, on_color_set,
	                         on_clip_set, on_clip_unset, NULL);

	rb_obj_call_init (self, 0, NULL);

	return self;
}

static void c_object_free (RbEvasObject *e)
{
	c_evas_object_free (e, true);
}

/*
 * call-seq:
 *  smart.new_object(class, evas) => smart_obj
 *
 * Creates a Evas::EvasObject from <i>smart</i>.
 */
static VALUE c_new_object (VALUE self, VALUE klass, VALUE evas)
{
	VALUE obj, argv[1];
	RbEvasObject *smart;

	CHECK_CLASS (evas, cEvas);
	GET_OBJ (evas, RbEvas, e);

	GET_OBJ (self, Evas_Smart *, s);

	obj = Data_Make_Struct (klass, RbEvasObject, c_evas_object_mark,
	                        c_object_free, smart);
	smart->real = evas_object_smart_add (e->real, *s);

	evas_object_data_set (smart->real, RUBY_EVAS_SMART_OBJECT_KEY,
	                      (void *) obj);

	argv[0] = evas;
	rb_obj_call_init (obj, 1, argv);

	return obj;
}

void Init_Smart (void)
{
	VALUE c = rb_define_class_under (mEvas, "Smart", rb_cObject);

	rb_define_singleton_method (c, "new", c_new, 1);
	rb_define_method (c, "new_object", c_new_object, 2);
}
