/*
 * $Id: rb_text.c 143 2004-11-26 21:39:08Z tilman $
 *
 * Copyright (C) 2004 Tilman Sauerbeck (tilman at code-monkey de)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ruby.h>

#include <Evas.h>

#include "rb_evas_main.h"
#include "rb_evas.h"
#include "rb_evas_object.h"

static void c_free (RbEvasObject *e)
{
	c_evas_object_free (e, true);
}

/*
 * call-seq:
 *  Evas::Text.new(evas) => text
 *
 * Creates an Evas::Text object.
 */
static VALUE c_new (VALUE klass, VALUE evas)
{
	VALUE self, argv[1];
	RbEvasObject *text;

	CHECK_CLASS (evas, cEvas);
	GET_OBJ (evas, RbEvas, e);

	self = Data_Make_Struct (klass, RbEvasObject, c_evas_object_mark,
	                         c_free, text);
	text->real = evas_object_text_add (e->real);

	argv[0] = evas;
	rb_obj_call_init (self, 1, argv);

	return self;
}

/*
 * call-seq:
 *  text.font_source => string or nil
 *
 * Returns the font source of <i>text</i>.
 */
static VALUE c_font_source_get (VALUE self)
{
	const char *tmp;

	GET_OBJ (self, RbEvasObject, e);

	if (!(tmp = evas_object_text_font_source_get (e->real)))
		return Qnil;
	else
		return rb_str_new2 (tmp);
}

/*
 * call-seq:
 *  text.font_source(source)
 *
 * Sets the font source of <i>text</i>.
 */
static VALUE c_font_source_set (VALUE self, VALUE val)
{
	GET_OBJ (self, RbEvasObject, e);

	Check_Type (val, T_STRING);

	evas_object_text_font_source_set (e->real, StringValuePtr (val));

	return Qnil;
}

/*
 * call-seq:
 *  text.get_font => array
 *
 * Returns the font name and font size of <i>text</i>.
 *
 *  text.set_font("vera", 10) #=> nil
 *  text_get_font             #=> ["vera", 10]
 */
static VALUE c_get_font (VALUE self)
{
	char *font = NULL;
	Evas_Font_Size size = 0;

	GET_OBJ (self, RbEvasObject, e);

	evas_object_text_font_get (e->real, &font, &size);

	return rb_ary_new3 (2, font ? rb_str_new2 (font) : Qnil,
	                    INT2FIX (size));
}

/*
 * call-seq:
 *  text.set_font(font, size) => nil
 *
 * Sets the font name and font size of <i>text</i>.
 *
 *  text.set_font("vera", 10) #=> nil
 */
static VALUE c_set_font (VALUE self, VALUE font, VALUE size)
{
	GET_OBJ (self, RbEvasObject, e);

	Check_Type (font, T_STRING);
	Check_Type (size, T_FIXNUM);

	evas_object_text_font_set (e->real, StringValuePtr (font),
	                           FIX2INT (size));

	return Qnil;
}

/*
 * call-seq:
 *  text.text => string
 *
 * Returns the text of <i>text</i>.
 */
static VALUE c_text_get (VALUE self)
{
	const char *tmp;

	GET_OBJ (self, RbEvasObject, e);

	if (!(tmp = evas_object_text_text_get (e->real)))
		return Qnil;
	else
		return rb_str_new2 (tmp);
}

/*
 * call-seq:
 *  text.text(string)
 *
 * Sets the text of <i>text</i>.
 */
static VALUE c_text_set (VALUE self, VALUE val)
{
	GET_OBJ (self, RbEvasObject, e);

	Check_Type (val, T_STRING);

	evas_object_text_text_set (e->real, StringValuePtr (val));

	return Qnil;
}

void Init_Text (void)
{
	VALUE c = rb_define_class_under (mEvas, "Text", cEvasObject);

	rb_define_singleton_method (c, "new", c_new, 1);
	rb_define_method (c, "font_source", c_font_source_get, 0);
	rb_define_method (c, "font_source=", c_font_source_set, 1);
	rb_define_method (c, "get_font", c_get_font, 0);
	rb_define_method (c, "set_font", c_set_font, 2);
	rb_define_method (c, "text", c_text_get, 0);
	rb_define_method (c, "text=", c_text_set, 1);
}
